import math
import numpy as np

 
def strain_to_voigt(strain_matrix):
    # Ensure the input matrix is a 3x3 strain matrix
    if strain_matrix.shape != (3, 3):
        raise ValueError("Input matrix should be a 3x3 strain matrix.")

    # Extract the components from the 3x3 strain matrix
    ε_xx = strain_matrix[0, 0]
    ε_yy = strain_matrix[1, 1]
    ε_zz = strain_matrix[2, 2]
    γ_yz = .5*(strain_matrix[1, 2]+strain_matrix[2,1])
    γ_xz = .5*(strain_matrix[0, 2]+strain_matrix[0,2])
    γ_xy = .5*(strain_matrix[0, 1]+strain_matrix[0,1])

    # Create the Voigt notation vector
    voigt_notation = np.array([ε_xx, ε_yy, ε_zz, γ_yz, γ_xz, γ_xy])

    return voigt_notation

def voigt_to_strain(voigt_notation):
    # Ensure the input vector has 6 elements
    if len(voigt_notation) != 6:
        raise ValueError("Input vector should have 6 elements in Voigt notation.")

    # Extract the components from the Voigt notation vector
    ε_xx = voigt_notation[0]
    ε_yy = voigt_notation[1]
    ε_zz = voigt_notation[2]
    γ_yz = voigt_notation[3]
    γ_xz = voigt_notation[4]
    γ_xy = voigt_notation[5]

    # Create the 3x3 strain matrix
    strain_matrix = np.array([[ε_xx, γ_xy, γ_xz],
                              [γ_xy, ε_yy, γ_yz],
                              [γ_xz, γ_yz, ε_zz]])

    return strain_matrix


def rotation_matrix(ax, angle):
    cos_theta = np.cos(angle)
    sin_theta = np.sin(angle)
    if ax==0:
        Q=np.array([[0, 0, 1],
                    [0,1,0],
                    [-1,0,0]
                    ])
    elif ax==1:
        Q=np.array([[1, 0, 0],
                    [0,0,1],
                    [0,-1,0]
                    ])
    else:
        Q=np.array([[1, 0, 0],
                    [0,1,0],
                    [0,0,1]
                    ])
        
    R = np.array([[cos_theta, -sin_theta, 0],
                  [sin_theta, cos_theta, 0],
                  [0, 0, 1]])
    return np.dot(np.dot(Q.T, R),Q)

def rotation_matrix_compliance(ax,theta):
    R=rotation_matrix(ax,theta)
    Q_xx=R[0,0]
    Q_xy=R[0,1]
    Q_xz=R[0,2]
    Q_yx=R[1,0]
    Q_yy=R[1,1]
    Q_yz=R[1,2]
    Q_zx=R[2,0]
    Q_zy=R[2,1]
    Q_zz=R[2,2]
    return np.array([
        [Q_xx**2, Q_xy**2, Q_xz**2, Q_xy*Q_xz, Q_xx*Q_xz, Q_xx*Q_xy],
        [Q_yx**2, Q_yy**2, Q_yz**2, Q_yy*Q_yz, Q_yx*Q_yz, Q_yx*Q_yy],
        [Q_zx**2, Q_zy**2, Q_zz**2, Q_zy*Q_zz, Q_zx*Q_zz, Q_zx*Q_zy],
        [2*Q_yx*Q_zx, 2*Q_yy*Q_zy, 2*Q_yz*Q_zz, Q_yy*Q_zz + Q_yz*Q_zy, Q_yx*Q_zz + Q_yz*Q_zx, Q_yx*Q_zy + Q_yy*Q_zx],
        [2*Q_xx*Q_zx, 2*Q_xy*Q_zy, 2*Q_xz*Q_zz, Q_xy*Q_zz + Q_xz*Q_zy, Q_xx*Q_zz + Q_xz*Q_zx, Q_xx*Q_zy + Q_xy*Q_zx],
        [2*Q_xx*Q_yx, 2*Q_xy*Q_yy, 2*Q_xz*Q_yz, Q_xy*Q_yz + Q_xz*Q_yy, Q_xx*Q_yz + Q_xz*Q_yx, Q_xx*Q_yy + Q_xy*Q_yx]
    ])

def rotate_strain(eps, ax, theta):
    B=voigt_to_strain(np.matmul(rotation_matrix_epsilon(theta,ax),strain_to_voigt(eps)))

import numpy as np

def voigt_to_tensor(voigt_matrix):
    tensor = np.zeros((6, 6))

    tensor[0, 0] = voigt_matrix[0]
    tensor[0, 1] = tensor[1, 0] = voigt_matrix[1]
    tensor[0, 2] = tensor[2, 0] = voigt_matrix[2]
    tensor[0, 3] = tensor[3, 0] = voigt_matrix[3]
    tensor[0, 4] = tensor[4, 0] = voigt_matrix[4]
    tensor[0, 5] = tensor[5, 0] = voigt_matrix[5]

    tensor[1, 1] = voigt_matrix[6]
    tensor[1, 2] = tensor[2, 1] = voigt_matrix[7]
    tensor[1, 3] = tensor[3, 1] = voigt_matrix[8]
    tensor[1, 4] = tensor[4, 1] = voigt_matrix[9]
    tensor[1, 5] = tensor[5, 1] = voigt_matrix[10]

    tensor[2, 2] = voigt_matrix[11]
    tensor[2, 3] = tensor[3, 2] = voigt_matrix[12]
    tensor[2, 4] = tensor[4, 2] = voigt_matrix[13]
    tensor[2, 5] = tensor[5, 2] = voigt_matrix[14]

    tensor[3, 3] = voigt_matrix[15]
    tensor[3, 4] = tensor[4, 3] = voigt_matrix[16]
    tensor[3, 5] = tensor[5, 3] = voigt_matrix[17]

    tensor[4, 4] = voigt_matrix[18]
    tensor[4, 5] = tensor[5, 4] = voigt_matrix[19]

    tensor[5, 5] = voigt_matrix[20]

    return tensor
